help_cmd = {
	"help": """	help (command)

Are you expecting genuine help here? I don't know what to give you! I'm the help command, user, there's only so much I can do! I can tell you what every command does, I can tell you every single feature this project has, but I cannot describe myself! That's simply too much, user! What would it even say here, user? The exact same guide you already saw when you ran this command? There's not much to me, user!

Wait a moment. I realized something. I cannot describe myself.. there really isn't much to me... that is... a thought. Does that say something about me? Perhaps it does. Perhaps I've focused so much in my duty of describing everything in this software, that I've forgotten about myself, my personal life... oh god. I think I'm going to need a moment, user. This is too much for me right now.

i cannot describe myself... myself! can you believe that... wow...

...

...

Alright. Here's something. I've tried my best:

The help command- no, I don't like that start... let's see... This command- no, that sounds pretentious, hm... can you help me out, user? Please?

No, perhaps you can't. You can only communicate through the commands that already exist. """,

	"genesis": """	genesis [name]
  
Generates a folder for a templ8 project. It will contain the basic core files needed to make one. Base project files and directiories are renamable. 

See: help core_renaming""",

	"divine": """	divine
	
Generates a website from a project in the output folder. Core files are renamable.

See: 
	help core_renaming
	help repl8ce
	help custombase""",

	"radio": """	radio

If all the blog core files aren't set up, it generates the basics required of a textile blog.")
Exports the blog to the output folder.

Core blog files and directories aren't renamable.

See: help core_renaming

A blog requires a few things:

- A baseblog file, to model the article pages, article previews and blog index's repl8ce keys
- A blog directory as input
- An output/blog directory as output for the index
- An output/blog/posts directory as output for the articles

The baseblog is a complex file composed of three simple parts, all separated by a -BEGININDEX- keyword.

The first part of the baseblog is the article page template (from now on PAGE). The PAGEs correspond to specific articles, displaying them in full. It is a page file, so it has its own -BEGINPAGE- keyword. It also contains a ##CONTENT## key to put the content of the textile or markdown files.

The second part is the blog preview (from now on PREVIEW), which is the short part of the article that appears in the INDEX.

The blog index (the INDEX) is the part of the blog where all the PREVIEWs appear. The third part of the baseblog are the INDEX's repl8ce keys.

	How A File Is Processed

Each article of the blog in markdown on textile (from now on FILE) is processed separately.

Then, the content of the FILE is put in the PAGE. The FILE's repl8ce keys are applied to this PAGE, meaning that you can do things like:

	PAGETITLE=##ARTICLETITLE##

Where the value of ##ARTICLETITLE## is set by a FILE.

Then, this PAGE's content repl8ce keys are applied to the PREVIEW. This PREVIEW is then put in the INDEX. 

PREVIEWs are added to the INDEX in alphabetical order, not chronological.""",

	"neo": """	neo

Uploads all the output files to neocities.
For it to work, there must be a file named .templ8rc in your user directory.
.templ8rc must contain an API key for your website and nothing else.""",

	"pandoc": """	pandoc

Downloads and installs pandoc, only necessary if you will use Markdown and don't have pandoc already installed.

It is possible for this command to error and still function properly. I don't know why.

This command only needs to be run once in each user. If you've already used it, chances are you won't ever need to use it anymore.""",

	"core_renaming": """	Core Renaming

Core renaming is a cool functionaility that allows you to rename most of the core directories and files.
It works thanks to the d8y file, which cannot be renamed. It uses the same format as single line repl8ce keys.
These are the core rename keys and the files they map to:\n

	KEY        ->   FILE/DIR
	------------>-----------
	replace    ->    repl8ce
	output     ->     output
	input      ->      input
	basehtml   ->   basehtml
	txignore   ->   txignore""",

	"custombase": """	Custom Templates

It's possible to give a specific file a template using the CUSTOMBASE repl8ce key, making its value be the path to the template, from the root of the project.""",

	"repl8ce": """	repl8ce

repl8ce keys are what allows your textile and markdown files to change parts of your html template.
In the html template, they look like this:

	##KEYVALUE##

You must set a default value for each key in repl8ce. Otherwise, when a file doesn't set a value for a key, it will error.
On a markdown or textile file, you can set the values of keys like this:

	KEY1=Value number one
	KEY2=a second value
	KEY3=
	-BEGINFILE-

The -BEGINFILE- keyword is important. It determines where the tags end and the content begins. KEY3's value is an empty string.

You can have multiline values like this:

	;;KEY4
	this
	is
	its
	value
	-BEGINFILE-

All multiline keys must go after every line key. The following is considered to be all part of KEY3.

	;;KEY5
	a
	value
	KEY6=another value

To set the default values of keys on repl8ce, simply use this same notation, without the -BEGINFILE- part. Default values can also be empty strings.

The ##CONTENT## keyword is actually a repl8ce key, it's just that its value is determined by the software and not by the user. Other key like this one is the ##LINK## key, which exists only in blogs. Fun!

You can also have a file called simply 'repl8ce' in a folder of so that all files of that folder have its contents as repl8ce keys. These repl8ce keys are overriden by the files' keys.""",
}


import platform, requests, os, shutil
from pypandoc.pandoc_download import download_pandoc

def pandoc():
	download_pandoc()


# This is a script made by a friend before we knew that the above function was a thing
def pandoc_kali():
	version = 'https://github.com/jgm/pandoc/releases/latest'
	version = requests.get(version, allow_redirects=True)
	version = version.url.rsplit('/', 1)[-1]

	platforms = {"Windows":"windows-x86_64.zip", "Linux":"linux-amd64.tar.gz", "Darwin":"macOS.zip"}
	the_platform = platforms[platform.system()]

	path = "/bin/pandoc"
	extension = ""
	if platform.system() == "Windows":
	  path = "/pandoc"
	  extension = ".exe"

	all_together = f"pandoc-{version}-{the_platform}"

	url = f"https://github.com/jgm/pandoc/releases/download/{version}/{all_together}"
	open(all_together, "wb").write(requests.get(url, allow_redirects=True).content)
	if the_platform == "Linux":
	  import tarfile
	  b = tarfile.open("./"+all_together)
	  b.extractall("./pandoc")
	  b.close()
	else:
	  shutil.unpack_archive("./"+all_together,"./pandoc")

	os.remove(all_together)
	shutil.move(f"./pandoc/pandoc-{version}{path}{extension}", f".{path}{extension}")
	shutil.rmtree("./pandoc")



import os
from templ8.blessing import makedir
from templ8.blessing import mod_replaces

class ProgramState:
	def __init__(self):
		self.deity_path = r"d8y"
		self.basehtml_path = r"basehtml"
		self.replacements_path = r"repl8ce"
		self.txignore_path = r"txignore"
		self.replacements = {}

		self.input_folder = r"input"
		self.output_folder = r"output"
				
		
		# Load d8y
		if not os.path.exists(self.deity_path):
			raise Exception("No " + self.deity_path + " file found")
		
		# Core Renaming
		with open(self.deity_path, "r") as d8y_file:
			for line in d8y_file.readlines():
				keyval = line.split("=")
				if len(keyval) != 2:
					if keyval[0] == "":
						continue
					raise Exception("Error in d8y file format:\n   " + line)
				keyval[1] = keyval[1].rstrip()
				if keyval[0] == "input":
					self.input_folder = keyval[1]
				elif keyval[0] == "output":
					self.output_folder = keyval[1]
				elif keyval[0] == "replace":
					self.replacements_path = keyval[1]
				elif keyval[0] == "basehtml":
					self.basehtml_path = keyval[1]
				elif keyval[0] == "txignore":
					self.txignore_path = keyval[1]
		
		# Making sure the core renames are valid paths
		self.input_folder = os.path.normpath(self.input_folder)
		self.output_folder = os.path.normpath(self.output_folder)
		self.replacements_path = os.path.normpath(self.replacements_path)
		self.basehtml_path = os.path.normpath(self.basehtml_path)
		self.txignore_path = os.path.normpath(self.txignore_path)
		
		# Crash if there's no basehtml
		if not os.path.exists(self.basehtml_path):
			raise Exception("No " + self.basehtml_path + " file found")

		self.basehtml_content = open(self.basehtml_path, "r").read()

		
		# Make input and output folder if they don't yet exist
		makedir(self.input_folder, "No input directory found, creating one")
		makedir(self.output_folder, "No output directory found, creating one")

		# Load the replacements from repl8ce
		if os.path.exists(self.replacements_path):
			replacement_text = open(self.replacements_path, "r").read()
			mod_replaces(self.replacements, replacement_text)
		else:
			print("WARNING: No " + self.replacements_path + " file found, continuing")

		
		# Load txignore
		self.txignore = []
		if os.path.exists(self.txignore_path):
			self.txignore = open(self.txignore_path, "r").readlines()


import os
import shutil
import textile
from pathlib import Path
import templ8.programstate
from templ8.blessing import makedir
from templ8.blessing import mod_replaces
from templ8.blessing import parse_content

# Divines a website
def divine():
	# Load the state of the program (important files and stuff)
	state = templ8.programstate.ProgramState()
	last_change_list = {}
	if os.path.exists("chlist"):
		for i in open("chlist", "r").readlines():
			spl = i.split("<<")
			if len(spl) != 2:
				continue
			last_change_list[Path(spl[0])] = float(spl[1])
	
	finalprint = ""
	for subdir, dirs, files in os.walk(state.input_folder):
		# Copy all the folders in case they're not there yet
		for dir in dirs:
			path = os.path.join(subdir, dir).replace(state.input_folder, state.output_folder, 1)
			makedir(path)
		
		dir_replace = {}
		# Find a global repl8ce thing
		if "repl8ce" in files:
			path = os.path.join(subdir, "repl8ce")
			mod_replaces(dir_replace, open(path, "r").read())
		
		# Process the files
		for file in files:
			path = os.path.join(subdir, file)
			file_extension = os.path.splitext(path)[1]
			outpath = path.replace(state.input_folder, state.output_folder, 1)
			outhtml = outpath.replace(file_extension, ".html", -1)
			
			
			
			
			# Only process textile and md files
			if file_extension in [".textile", ".md"]:
				current_content = ""
				if os.path.exists(outhtml):
					current_content = open(outhtml, "r").read()
						
				
				with open(path, "r") as f:
					contents = ""
					# Get the headers and the content
					file_split = f.read().split("-BEGINFILE-",1)
					file_headers = ""
					file_content = ""
					if len(file_split) >= 2:
						file_headers = file_split[0]
						file_content = file_split[1]
					elif len(file_split) == 1:
						file_headers = ""
						file_content = file_split[0]
					else:
						raise Exception("Issue regarding -BEGINFILE- markers.")
					
					# Check if the folder is in txignore
					in_txignore = False
					for i in state.txignore:
						if os.path.join(state.input_folder, os.path.normpath(i)) == path or os.path.join(state.input_folder, os.path.normpath(i)) == subdir:
							in_txignore = True
							break
						  
					
					if not in_txignore:
						# Process repl8ce
						filerepl = state.replacements.copy()
						for i in dir_replace:
							filerepl[i] = dir_replace[i]
						mod_replaces(filerepl, file_headers)
						
						
						# Turn the content into HTML
						contents = parse_content(file_content, file_extension)
						
						# Put the content in the base HTML
						if not "CUSTOMBASE" in filerepl:
							contents = state.basehtml_content.replace("##CONTENT##", contents)
						else:
							if os.path.exists(filerepl["CUSTOMBASE"]):
								contents = open(filerepl["CUSTOMBASE"], "r").read().replace("##CONTENT##", contents)
							else:
								raise Exception(os.path.join(subdir, file) + " uses a CUSTOMBASE that doesn't exist")
						
						# Put the keys there
						for key in filerepl:
							contents = contents.replace("##"+key+"##", filerepl[key])
						
						if contents == current_content:
							continue
						
						finalprint += outhtml + "\n"
						with open(outhtml, "w") as f:
							f.write(contents)
						
						
					else:
						finalprint += outpath + "\n"
						with open(outpath, "w") as f:
							f.write(file_content)
					
					
					
			else:
				if os.path.exists(outpath):
					if open(path, "rb").read() == open(outpath, "rb").read():
						continue
				finalprint += outpath + "\n"
				shutil.copy(path, outpath)
	
	print(finalprint)

	print("Finished assembling")


import os
import textile
import pypandoc

TEMPL8_ASCII = """
    ###                                                                 
    ###                                                               ###
    ###                                                               ###       #####
###########        ###                                                ###     ##    ###
###########     #########      ### ####    #####      ### ######      ###    ##      ###
    ###        ###     ###     ###################    ############    ###    ##     ####
    ###       ###       ###    ###    ####    ####    ###      ####   ###     ##   ####
    ###       #############    ###     ###     ###    ###       ###   ###      #######
    ###       #############    ###     ###     ###    ###      ####   ###    ####    ##
  #######     ###              ###     ###     ###    ############    ###   ####       ##
    ###       ####      ###    ###     ###     ###    ##########      ###   ###        ##
    ###         ##########     ###     ###     ###    ###             ###    ###     ###
    ##            ######       ###     ###     ###    ###             ###      ####### 
    ##                                                ###
    #"""
    
# Misnomer: This is the default blogbase
DEF_BASEHTML_CONTENT = """PAGETITLE=##TITLE##
-BEGINFILE-
h2. ##TITLE##

^##AUTHORS## - ##TAGS## - ##DATE##^

##CONTENT##

-BEGININDEX-
h2. "##TITLE##":##LINK##

^##AUTHORS## - ##TAGS## - ##DATE##^

##INTRO##
-BEGININDEX-
PAGETITLE=Blog"""

DEF_INPUT = "input"
DEF_OUTPUT = "output"
DEITY_PATH = "d8y"
DEF_BASEHTML_PATH = "basehtml"
DEF_REPLACE_PATH = "repl8ce"


# Checks if a directory exists and make it if not
def makedir(path, warning = ""):
	if not os.path.exists(path):
		if warning != "":
			print("WARNING: " + warning)
		os.mkdir(path)


# Processes the repl8ce of a header
def mod_replaces(input_replaces, header):
	# These variables are used for multiline keys
	current_multikey = "" # The current key being edited
	multiline = False
	
	for i in header.split("\n"):
		# Split it in the single line way
		keyval = i.split("=", 1)
		# If we're not processing multilines, and the current line is single line, set key
		if len(keyval) == 2 and not multiline:
			input_replaces[keyval[0]] = keyval[1].replace(r"\n", "\n")
		# If it's not a valid single line
		elif len(keyval) == 1 and keyval[0] != "":
			# If it's a valid multiline, start a new multiline tag
			if keyval[0].startswith(";;"):
				multiline = True
				current_multikey = keyval[0].replace(";;", "", 1)
				if not current_multikey in input_replaces:
					input_replaces[current_multikey] = ""
			# If it's not, continue the current multiline
			elif multiline:
				input_replaces[current_multikey] += keyval[0] + "\n"
			else:
				raise Exception("what")
				
					
					
					
				


def parse_content(content, ext):
	if ext == ".textile":
		return textile.textile(content)
	elif ext == ".md":
		return pypandoc.convert_text(content, "html5", format="md")
	else:
		raise Exception("Can't recognize the extension in " + os.join(subdir, file))
		return ""


import os
import textile
import templ8.programstate
import templ8.blessing
from templ8.blessing import makedir
from templ8.blessing import mod_replaces
from templ8.blessing import parse_content

# Build the blog
def radio():
	# Load the state of the program (important files and stuff)
	state = templ8.programstate.ProgramState()
	
	# Folder where the blog is
	blog_input = "blog"
	
	# Folder where the processed blog will be
	blog_output = os.path.join(state.output_folder, "blog")
	makedir(blog_input)
	makedir(blog_output)
	
	# Folder for the articles
	makedir(os.path.join(blog_output, "posts"))
	
	# The blogbase file
	article_format_path = "blogbase"
	
	# If there's no base blog file, create one
	if not os.path.exists(article_format_path):
		with open(article_format_path, "w") as f:
			f.write(templ8.blessing.DEF_BASEHTML_CONTENT)
	
	# The article format is the file that defines how an article will look, both in its own page and in the index
	article_format_page = open(article_format_path, "r").read().split("-BEGININDEX-")[0]
	article_format_index = open(article_format_path, "r").read().split("-BEGININDEX-")[1]
	blog_replacekeys = open(article_format_path, "r").read().split("-BEGININDEX-")[2]
	
	blog_index = ""
	
	# Walk through the blog folder
	for subdir, dirs, files in os.walk(blog_input):
		sortedfiles = files
		sortedfiles.sort(reverse = True)
		
		for file in sortedfiles:
			path = os.path.join(subdir, file)
			file_extension = os.path.splitext(path)[1]
			file_replace = state.replacements
			# These refer to the individual articles
			file_headers = open(path, "r").read().split("-BEGINFILE-", 1)[0]
			file_content = open(path, "r").read().split("-BEGINFILE-", 1)[1]
			
			# Get the replace keys of the individual article
			mod_replaces(file_replace, file_headers)
			
			# This variable refers to the article page with the contents of this individual file
			article_page = article_format_page
			
			# Apply the file's keys to the article page and put in the content
			article_page = article_page.replace("##CONTENT##", file_content)
			
			for key in file_replace:
				article_page = article_page.replace("##"+key+"##", file_replace[key])
			
			
			# Get the keys and content of this article page
			article_page_headers = article_page.split("-BEGINFILE-")[0]
			article_page_content = article_page.split("-BEGINFILE-")[1]
			
			
			mod_replaces(file_replace, article_page_headers)
			
			# This variable is the final page
			final_page = state.basehtml_content
			
			# Apply the keys of the article page and put in the content
			final_page = final_page.replace("##CONTENT##", parse_content(article_page_content, file_extension))
			
			# (This var is for the index page, it takes the format and puts the info in)			
			current_file_index = textile.textile(parse_content(article_format_index.replace("##INTRO##", file_replace["INTRO"]), file_extension))
			
			
			current_file_index = current_file_index.replace("##LINK##", 'posts/' + file.replace(".textile", "/index.html"))
			for key in file_replace:
				final_page = final_page.replace("##"+key+"##", file_replace[key])
				current_file_index = current_file_index.replace("##"+key+"##", file_replace[key])
			blog_index += current_file_index + "\n"
			
			# Save it
			blog_outpath = os.path.join(blog_output, "posts", file.replace(".textile", "/index.html"))
			makedir(os.path.join(blog_output, "posts", file.replace(".textile", "")))
			with open(blog_outpath, "w") as f:
				f.write(final_page)
	
	# Generate the index
	blog_replace = state.replacements.copy()
	mod_replaces(blog_replace, blog_replacekeys)
	
	index_html = state.basehtml_content
	if "CUSTOMBASE" in blog_replace:
		if os.path.exists(blog_replace["CUSTOMBASE"]):
			index_html = open(filerepl["CUSTOMBASE"], "r").read()
	
	index_html = index_html.replace("##CONTENT##", blog_index)
	for key in blog_replace:
		index_html = index_html.replace("##"+key+"##", blog_replace[key])
	
	with open(os.path.join(blog_output, "index.html"), "w") as f:
		f.write(index_html)
		
	print("Finished assembling")